@@ -249,6 +249,7 @@
 #include <algorithm>
 
 #include <dune/common/fvector.hh>
+#include <dune/common/power.hh>
 
 #include <dune/geometry/multilineargeometry.hh>
 #include <dune/geometry/referenceelements.hh>
@@ -584,7 +585,7 @@ namespace Dune {
       {
         std::vector<CoordVector> corners(1);
         corners[0] = (CoordVector)vertex;
-        return Geometry(GeometryType(0), corners);
+        return Geometry(GeometryTypes::vertex, corners);
       }
 
       template<int dimension, class CoordType>
@@ -737,7 +738,6 @@ namespace Dune {
       RefinementIteratorSpecial<dimension, CoordType, 0>::geometry () const
       {
         std::vector<CoordVector> corners(dimension+1);
-        CoordVector v;
         auto refelem = ReferenceElements<CoordType, dimension>::simplex();
         for(int i = 0; i <= dimension; ++i)
           corners[i] = global(refelem.position(i, dimension));
@@ -798,9 +798,9 @@ namespace Dune {
     struct Traits<
         topologyId, CoordType, coerceToId, dim,
         typename std::enable_if<
-            ((Impl::SimplexTopology<dim>::type::id >> 1) ==
+            ((GeometryTypes::simplex(dim).id() >> 1) ==
              (topologyId >> 1) &&
-             (Impl::SimplexTopology<dim>::type::id >> 1) ==
+             (GeometryTypes::simplex(dim).id() >> 1) ==
              (coerceToId >> 1)
             )>::type
         >
